#include "ofApp.h"

//--------------------------------------------------------------
void ofApp::setup(){
	width = 320; height = 240;
	vidGrabber.initGrabber(width, height); //abre cmera

	colorImage.allocate(width,height); //aloca memria para as imagens
	grayBg.allocate(width,height);
	grayImage.allocate(width,height);
	grayDiff.allocate(width,height);

	contourFinder.setMinAreaRadius(10); //configura rastreador
	contourFinder.setMaxAreaRadius(150);
	contourFinder.setTargetColor(ofColor::white, TRACK_COLOR_RGB);

	threshold = 50;
	bLearnBakground = true;

}

//--------------------------------------------------------------
void ofApp::update(){
	vidGrabber.update(); // Pega frame da cmera
	if(vidGrabber.isFrameNew()){ //  um frame novo ???
		colorImage.setFromPixels(vidGrabber.getPixels(),width,height);
		grayImage = colorImage;

		if (bLearnBakground == true){
			grayBg = grayImage;// salva o fundo da tela - TO ROUBANDO MESMO!
			bLearnBakground = false;}

		grayDiff.absDiff(grayBg, grayImage); // fazer fundo x atual
		grayDiff.threshold(threshold);

		contourFinder.setThreshold(threshold); // procurar blobs no atual
		contourFinder.findContours(grayDiff);
		apontador = encontraPontoMaisDistante(); // procurar dedo
	} 

}

ofPoint ofApp::encontraPontoMaisDistante(){
	int n = contourFinder.size(); //qtidade de blobs capturados
	ofPoint maisDistante ; //nosso ponto mais distante
	for(int i = 0; i < n; i++) {
		ofVec2f centroid = toOf(contourFinder.getCentroid(i)); //Centroide;
		ofPolyline convexHull = toOf(contourFinder.getConvexHull(i)); //quinas;
		vector<ofPoint> vertices = convexHull.getVertices(); // vetorDeQuinas
		vector<ofPoint>::iterator itVec; // para percorrer as quinas.
		maisDistante = (*vertices.begin());
		float tamMaisDistante = centroid.distanceSquared(maisDistante);
		float distanciaAtual = 0;
		for(itVec = vertices.begin(); itVec != vertices.end(); itVec++){
			//encontrar ponto mais distante do dedo --> eliminar as bordas
			if( ((*itVec).x > 40) && ((*itVec).x < width - 40) && 
				((*itVec).y > 40) && ((*itVec).y < height - 40)){ //elimina bordas
					distanciaAtual = centroid.distanceSquared((*itVec));
					if(distanciaAtual > tamMaisDistante){ //maior simple as possible
						maisDistante = (*itVec);
						tamMaisDistante = distanciaAtual;
					}
			}
		}
	}
	return maisDistante;
}


//--------------------------------------------------------------
void ofApp::draw(){
	ofSetColor(255); grayImage.draw(0, 0); // desenha cmera
	ofTranslate(320, 0); grayDiff.draw(0, 0); // desenha threshould 
	ofTranslate(-320,240); contourFinder.draw(); //desenha contorno encontrado

	ofSetColor(ofColor::hotPink);
	ofDrawBitmapString("openCV Threshold: " + ofToString(threshold),50,50);

	ofTranslate(0,-240);

	ofFill();
	ofSetColor(ofColor::green); 
	ofCircle(apontador,5); //desenha bola verde no dedo

}

//--------------------------------------------------------------
void ofApp::keyPressed(int key){

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){

	if ( key == 'L' || key == 'l')
		bLearnBakground = true;

}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}
